
# import __main__ as main
# def is_interactive():
#     return not hasattr(main, '__file__')
from .lazy_modules import LazyObject
import os
import os.path as osp
import time
from collections import defaultdict
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
from functools import partial, wraps
from glob import glob
from multiprocessing import Pool

from fastcore.all import *
from fastcore.parallel import threaded
from fastcore.script import *
from fastcore.script import Param, call_parse
from loguru import logger

import inspect
import json
from PIL import Image
from tqdm import tqdm
from pycocotools.coco import COCO
import matplotlib.pyplot as plt

mmcv = LazyObject('mmcv')
np = LazyObject('numpy')
ipdb = LazyObject('ipdb')
cv2 = LazyObject('cv2')


# from avcv.utils import memoize



class LazyObject(object):
    # ATTRS = dict()
    def __init__(self, module_name):
        self.module_name = module_name
        self.is_loaded = False
        self.tree = dict()
    def __getattr__(self, item):
        try:
            return self.tree[item]
        except:
            real_module = self.get_real_module()
            for obj_name in dir(real_module):
                self.tree[obj_name] = getattr(real_module, obj_name)
            return self.tree[obj_name]

    def get_real_module(self):
        if not 'real_module' in self.tree:
            exec(f'import {self.module_name}')
            self.tree['real_module'] =  eval(self.module_name)
        return self.tree['real_module']

    def __dir__(self):
        return self.tree.keys()

    def __repr__(self):
        real_module = self.get_real_module()
        return real_module.__repr__()
    def __help__(self):
        real_module = self.get_real_module()
        return help(real_module)